/*
 *
 * Copyright (c) 1998 Scott Oaks. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.security.cert.*;
import java.security.Signature;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.util.zip.*;
import java.io.*;
import java.util.*;

public class SignJar {
	public static void main(String args[]) {
		try {
			Vector classNames = new Vector();
			ZipEntry ze;
			KeyStore ks = KeyStore.getInstance();							   
			ks.load(new FileInputStream(System.getProperty("user.home") + File.separator + ".keystore"), args[1]);
			Certificate certs[] = ks.getCertificateChain(args[0]);
			PrivateKey pk = ks.getPrivateKey(args[0], args[1]);
			Signature sig = Signature.getInstance("DSA");
			sig.initSign(pk);

			ZipFile zf = new ZipFile(args[2]);
			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(args[2] + ".sig"));

			for (Enumeration e = zf.entries(); e.hasMoreElements(); ) {
				ze = (ZipEntry) e.nextElement();
				System.out.println("processing " + ze.getName());
				InputStream zipBuf = zf.getInputStream(ze);
				ByteArrayOutputStream zipOut = new ByteArrayOutputStream();
				int n;
				while ((n = zipBuf.read()) != -1) {
					zipOut.write(n);
				}
				byte data[] = zipOut.toByteArray();
				if (ze.getName().endsWith(".class")) {
					classNames.addElement(ze.getName());
					sig.update(data);
				}
				zos.putNextEntry(ze);
				zos.write(data, 0, data.length);
				zos.closeEntry();
			}

			ze = new ZipEntry("SIGNATURE");
			zos.putNextEntry(ze);
			byte signature[] = sig.sign();
			ObjectOutputStream oos = new ObjectOutputStream(zos);
			oos.writeObject(signature);
			zos.closeEntry();

			ze = new ZipEntry("CERTIFICATE");
			zos.putNextEntry(ze);
			oos = new ObjectOutputStream(zos);
			oos.writeObject(certs[0].getEncoded());
			zos.closeEntry();

			ze = new ZipEntry("CLASSNAMES");
			zos.putNextEntry(ze);
			oos = new ObjectOutputStream(zos);
			String classNameArray[] = new String[classNames.size()];
			classNames.copyInto(classNameArray);
			oos.writeObject(classNameArray);
			zos.closeEntry();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
